library ac_dart.descriptor_model.bitbucket.webhooks;

import 'package:stuff/stuff.dart';

import '../descriptor_model.dart';

class BitbucketWebhookIdentifier extends WebhookEventIdentifier {
  BitbucketWebhookIdentifier._(String key) : super(key);

  BitbucketWebhookIdentifier._std(String group, [String name])
      : this._(name != null ? '$group:$name' : group);

  BitbucketWebhookIdentifier._created(String group)
      : this._std(group, 'created');
  BitbucketWebhookIdentifier._updated(String group)
      : this._std(group, 'updated');
  BitbucketWebhookIdentifier._deleted(String group)
      : this._std(group, 'removed');
}

/*
pullrequest:created
 */
final _BitbucketEvents bitbucketWebhooks = new _BitbucketEvents._();

class _BitbucketEvents {
  final _PageEvents page = new _PageEvents._();
  final _PullRequestEvents pullRequest = new _PullRequestEvents._();

  _BitbucketEvents._();
}

class _PullRequestEvents {
  static const String pullRequest = 'pullrequest';
  final BitbucketWebhookIdentifier created =
      new BitbucketWebhookIdentifier._created(pullRequest);
  final BitbucketWebhookIdentifier updated =
      new BitbucketWebhookIdentifier._updated(pullRequest);
  final BitbucketWebhookIdentifier deleted =
      new BitbucketWebhookIdentifier._deleted(pullRequest);

  _PullRequestEvents._();
}

class _PageEvents {
  static const String page = 'page';
  final BitbucketWebhookIdentifier created =
      new BitbucketWebhookIdentifier._created(page);
  final BitbucketWebhookIdentifier updated =
      new BitbucketWebhookIdentifier._updated(page);
  final BitbucketWebhookIdentifier deleted =
      new BitbucketWebhookIdentifier._deleted(page);

  _PageEvents._();
}

abstract class BitbucketPRWebHookEvent implements Jsonable {
//  final BitbucketUser user;
  final BitbucketPRWebHookEventData data;

  factory BitbucketPRWebHookEvent.fromJson(Map json) =
      _BitbucketPRWebHookEvent.fromJson;
}

class _BitbucketPRWebHookEvent extends DynamicJson
    implements BitbucketPRWebHookEvent {
  BitbucketPRWebHookEventData get data =>
      parseSingle('data', (v) => new _BitbucketPRWebHookEventData.fromJson(v));

  _BitbucketPRWebHookEvent.fromJson(Map<String, Object> json)
      : super.fromJson(json);
}

abstract class BitbucketPRWebHookEventData implements Jsonable {
//  final BitbucketUser user;
  final BitbucketPullRequest pullRequest;

  factory BitbucketPRWebHookEventData.fromJson(Map json) =
      _BitbucketPRWebHookEventData.fromJson;
}

class _BitbucketPRWebHookEventData extends DynamicJson
    implements BitbucketPRWebHookEventData {
  BitbucketPullRequest get pullRequest =>
      parseSingle('pullrequest', (v) => new _BitbucketPullRequest.fromJson(v));

  _BitbucketPRWebHookEventData.fromJson(Map<String, Object> json)
      : super.fromJson(json);
}

abstract class BitbucketPullRequest implements Jsonable {
  final String title, description;
//  , creatorKey, creatorName, lastModifierKey, spaceKey;
  final int id;

  final DateTime creationDate;
//  , modificationDate;

  final BitbucketUser author;

  factory BitbucketPullRequest.fromJson(Map json) =
      _BitbucketPullRequest.fromJson;
}

class _BitbucketPullRequest extends DynamicJson
    implements BitbucketPullRequest {
  DateTime get creationDate => parseSingle('created_on', _toDate);
//  DateTime get modificationDate => parseSingle('modificationDate', _toDate);
  BitbucketUser get author =>
      parseSingle('author', (v) => new _BitbucketUser.fromJson(v));
  DateTime _toDate(String value) => DateTime.parse(value);

  _BitbucketPullRequest.fromJson(Map<String, Object> json)
      : super.fromJson(json);
}

abstract class BitbucketUser implements Jsonable {
  String get username;
}

class _BitbucketUser extends DynamicJson implements BitbucketUser {
  _BitbucketUser.fromJson(Map<String, Object> json) : super.fromJson(json);
}

//"author": {
//"username": "aholmgren",
//"type": "user",
//"display_name": "Anders Holmgren",
//"uuid": "{2d0e2033-16c0-4961-a185-c68ab036d0ca}",
//},

/*
{
  "data": {
    "pullrequest": {
      "type": "pullrequest",
      "description": "README.md edited online with Bitbucket",
      },
      "title": "README.md edited online with Bitbucket",
      "close_source_branch": true,
      "reviewers": [],
      "state": "OPEN",
      "id": 2,
      "source": {
        "commit": {
          "hash": "4e889fd9a14a",
        },
        "branch": {
          "name": "aholmgren/readmemd-edited-online-with-bitbucket-1449806340003"
        },
        "repository": {
          "full_name": "aholmgren/atlassian_connect_host",
          "type": "repository",
          "name": "atlassian_connect_host",
          "uuid": "{8f5802a2-741d-4768-b845-06b5df2ee17a}"
        }
      },
      "reason": "",
      "author": {
        "username": "aholmgren",
        "type": "user",
        "display_name": "Anders Holmgren",
        "uuid": "{2d0e2033-16c0-4961-a185-c68ab036d0ca}",
      },
      "created_on": "2015-12-11T03:59:04.245947+00:00",
      "participants": [],
      "updated_on": "2015-12-11T03:59:04.271325+00:00",
      "merge_commit": null,
      "closed_by": null
    },
    "actor": {
      "username": "aholmgren",
      "type": "user",
      "display_name": "Anders Holmgren",
      "uuid": "{2d0e2033-16c0-4961-a185-c68ab036d0ca}",
    },
    "repository": {
      "scm": "git",
      "name": "atlassian_connect_host",
      },
      "full_name": "aholmgren/atlassian_connect_host",
      "owner": {
        "username": "aholmgren",
        "type": "user",
        "display_name": "Anders Holmgren",
        "uuid": "{2d0e2033-16c0-4961-a185-c68ab036d0ca}",
      },
      "type": "repository",
      "is_private": false,
      "uuid": "{8f5802a2-741d-4768-b845-06b5df2ee17a}"
    }
  },
  "event": "pullrequest:created"
}
 */
