library ac_dart.descriptor_model.confluence;

import 'dart:convert';

import 'package:atlassian_connect_models/src/descriptor/descriptor_model.dart';
import 'package:constrain/constrain.dart';
import 'package:option/option.dart';
import 'package:quiver/iterables.dart';
import 'package:stuff/stuff.dart';

import 'scopes.dart';

class _ProductlessScope extends Scope implements ConfluenceScope {
  _ProductlessScope(String key) : super(key);
}

class ConfluenceConnectAddon
    extends ConnectAddon<ConfluenceModuleList, ConfluenceScope> {
  ConfluenceConnectAddon(
      {String key,
      String name,
      String description,
      ConfluenceModuleList modules,
      Uri baseUrl,
      String baseUrlStr,
      Authentication authentication,
      Lifecycle lifecycle,
      Vendor vendor,
      Iterable<ConfluenceScope> scopes})
      : super(
            key: key,
            name: name,
            description: description,
            modules: modules,
            baseUrl: baseUrl,
            baseUrlStr: baseUrlStr,
            authentication: authentication,
            lifecycle: lifecycle,
            vendor: vendor,
            scopes: scopes);

  ConfluenceConnectAddon.fromJson(Map json)
      : super.fromJson(json, (Map j) => new ConfluenceModuleList.fromJson(j),
            (String s) => new _ProductlessScope(s));

//  Map toJson() => super.toJson();

  String toString() => JSON.encode(this);
}

class ConfluenceModuleTypes {
  static const String dynamicContentMacro = 'dynamicContentMacro';
}

bool _oauthConsumerMandatoryForConfluence(Option<OAuthConsumer> consumer) =>
    consumer is Some;

class ConfluenceModuleList extends CoreModuleList {
  Iterable<DynamicContentMacro> get dynamicContentMacros =>
      filtered(ConfluenceModuleTypes.dynamicContentMacro);
  @Ensure(_oauthConsumerMandatoryForConfluence,
      description: 'oauth consumer is mandatory for confluence')
  Option<OAuthConsumer> get oauthConsumer => super.oauthConsumer;

  ConfluenceModuleList(
      {Iterable<Page> generalPages: const [],
      Iterable<Page> adminPages: const [],
      Iterable<WebItem> webItems: const [],
      Iterable<WebPanel> webPanels: const [],
      Page configurePage,
      Iterable<Webhook> webhooks: const [],
      Iterable<DynamicContentMacro> dynamicContentMacros: const [],
      OAuthConsumer oauthConsumer})
      : super.fromModules(concat([
          CoreModuleList.createCoreModules(generalPages, adminPages, webItems,
              webPanels, configurePage, webhooks, oauthConsumer),
          BaseModuleList.mapModules(
              dynamicContentMacros, ConfluenceModuleTypes.dynamicContentMacro)
        ]));

  ConfluenceModuleList.fromModules(Iterable<ClassifiedModule> modules)
      : super.fromModules(modules);

  ConfluenceModuleList.fromCore(
      {CoreModuleList coreModules,
      Iterable<DynamicContentMacro> dynamicContentMacros})
      : super.fromModules(concat([
          coreModules.modules,
          BaseModuleList.mapModules(
              dynamicContentMacros, ConfluenceModuleTypes.dynamicContentMacro)
        ]));

  factory ConfluenceModuleList.fromJson(Map json) {
    final p = parseJson(json);
    return new ConfluenceModuleList(
        generalPages: p.list('generalPage', (v) => new Page.fromJson(v)),
        adminPages: p.list('adminPage', (v) => new Page.fromJson(v)),
        webItems: p.list('webItem', (v) => new WebItem.fromJson(v)),
        webhooks: p.list('webhook', (v) => new Webhook.fromJson(v)),
        dynamicContentMacros: p.list(
            'dynamicContentMacros', (v) => new DynamicContentMacro.fromJson(v)),
        configurePage: p.single('configurePage', (v) => new Page.fromJson(v)));
  }

  Map toJson() => (buildJson
        ..addAll(super.toJson())
        ..add('dynamicContentMacros', dynamicContentMacros))
      .json;
}

abstract class BaseContentMacro implements KeyedModule {}

abstract class DynamicContentMacro extends BaseContentMacro {
  final String width, height;

  DynamicContentMacro(this.width, this.height);

  factory DynamicContentMacro.fromJson(Map<String, Object> json) =
      DynamicContentMacroImpl.fromJson;
}

class DynamicContentMacroImpl extends DynamicJson
    implements DynamicContentMacro {
  DynamicContentMacroImpl.fromJson(Map<String, Object> json)
      : super.fromJson(json);
}

abstract class BaseContentMacroImpl extends DynamicJson
    implements BaseContentMacro {
  BaseContentMacroImpl.fromJson(Map<String, Object> json)
      : super.fromJson(json);
}

/*
class DynamicContentMacro extends BaseContentMacro
{
    final String width, height;

//    final MacroRenderModes renderModes;
}

class BaseContentMacro {
    final UriTemplate url;

    final I18nProperty description;

    private IconBean icon;

    private LinkBean documentation;

    private Set<String> categories;

    @CommonSchemaAttributes(defaultValue = "block")
    private MacroOutputType outputType;

    @CommonSchemaAttributes(defaultValue = "none")
    private MacroBodyType bodyType;

    private Set<String> aliases;

    @CommonSchemaAttributes(defaultValue = "false")
    private Boolean featured;

    @CommonSchemaAttributes(defaultValue = "false")
    private Boolean hidden;

    private List<MacroParameterBean> parameters;

    private AutoconvertBean autoconvert;

    private MacroEditorBean editor;

    private ImagePlaceholderBean imagePlaceholder;
}
 */
