library ac_dart.descriptor_model.confluence.webhooks;

import 'package:atlassian_connect_models/src/descriptor/descriptor_model.dart';
import 'package:stuff/stuff.dart';

import '../descriptor_model.dart';

class ConfluenceWebhookIdentifier extends WebhookEventIdentifier {
  ConfluenceWebhookIdentifier._(String key) : super(key);

  ConfluenceWebhookIdentifier._std(String group, [String name])
      : this._(name != null ? '${group}_$name' : group);

  ConfluenceWebhookIdentifier._created(String group)
      : this._std(group, 'created');
  ConfluenceWebhookIdentifier._updated(String group)
      : this._std(group, 'updated');
  ConfluenceWebhookIdentifier._deleted(String group)
      : this._std(group, 'removed');
}

/*
page_created
page_moved
page_removed
page_restored
page_trashed
page_updated
page_viewed
 */
final _ConfluenceEvents confluenceWebhooks = new _ConfluenceEvents._();

class _ConfluenceEvents {
  final _PageEvents page = new _PageEvents._();

  _ConfluenceEvents._();
}

class _PageEvents {
  static const String page = 'page';
  final ConfluenceWebhookIdentifier created =
      new ConfluenceWebhookIdentifier._created(page);
  final ConfluenceWebhookIdentifier updated =
      new ConfluenceWebhookIdentifier._updated(page);
  final ConfluenceWebhookIdentifier deleted =
      new ConfluenceWebhookIdentifier._deleted(page);

  _PageEvents._();
}

abstract class ConfluenceWebHookEvent implements Jsonable {
//  final ConfluenceUser user;
  final ConfluencePage page;

  factory ConfluenceWebHookEvent.fromJson(Map json) =
      _ConfluenceWebHookEvent.fromJson;
}

class _ConfluenceWebHookEvent extends DynamicJson
    implements ConfluenceWebHookEvent {
  ConfluencePage get page =>
      parseSingle('page', (v) => new _ConfluencePage.fromJson(v));

  _ConfluenceWebHookEvent.fromJson(Map<String, Object> json)
      : super.fromJson(json);
}

abstract class ConfluencePage implements Jsonable {
  final String title, creatorKey, creatorName, lastModifierKey, spaceKey;
  final int id;

  final DateTime creationDate, modificationDate;

  ConfluencePage(
      this.title,
      this.creatorKey,
      this.creatorName,
      this.lastModifierKey,
      this.spaceKey,
      this.id,
      this.creationDate,
      this.modificationDate);
}

class _ConfluencePage extends DynamicJson implements ConfluencePage {
  DateTime get creationDate => parseSingle('creationDate', _toDate);
  DateTime get modificationDate => parseSingle('modificationDate', _toDate);

  DateTime _toDate(int value) => new DateTime.fromMillisecondsSinceEpoch(value);

  _ConfluencePage.fromJson(Map<String, Object> json) : super.fromJson(json);
}
/*
{
  "page": {
    "spaceKey": "ds",
    "modificationDate": 1449693636452,
    "creatorKey": "2c9682714db22c7c014db22f51970002",
    "creatorName": "admin",
    "lastModifierKey": "2c9682714db22c7c014db22f51970002",
    "self": "http://blah:1990/confluence/display/ds/abc",
    "lastModifierName": "admin",
    "id": 884739,
    "title": "abc",
    "creationDate": 1449693636452,
    "version": 1
  },
  "user": "admin",
  "userKey": "2c9682714db22c7c014db22f51970002",
  "timestamp": 1449693636490,
  "username": "admin"
}
 */
