library ac_dart.descriptor_model.jira;

import 'dart:convert';

import 'package:atlassian_connect_models/src/descriptor/descriptor_model.dart';
import 'package:constrain/constrain.dart';
import 'package:option/option.dart';
import 'package:quiver/iterables.dart';
import 'package:stuff/stuff.dart';

import 'scopes.dart';

class _ProductlessScope extends Scope implements JiraScope {
  _ProductlessScope(String key) : super(key);
}

class JiraConnectAddon extends ConnectAddon<JiraModuleList, JiraScope> {
  JiraConnectAddon(
      {String key,
      String name,
      String description,
      JiraModuleList modules,
      Uri baseUrl,
      String baseUrlStr,
      Authentication authentication,
      Lifecycle lifecycle,
      Vendor vendor,
      Iterable<JiraScope> scopes})
      : super(
            key: key,
            name: name,
            description: description,
            modules: modules,
            baseUrl: baseUrl,
            baseUrlStr: baseUrlStr,
            authentication: authentication,
            lifecycle: lifecycle,
            vendor: vendor,
            scopes: scopes);

  JiraConnectAddon.fromJson(Map json)
      : super.fromJson(json, (Map j) => new JiraModuleList.fromJson(j),
            (String s) => new _ProductlessScope(s));

//  Map toJson() => super.toJson();

  String toString() => JSON.encode(this);
}

class JiraModuleTypes {
  static const String repoPage = 'repoPage';
}

bool _oauthConsumerMandatoryForJira(Option<OAuthConsumer> consumer) =>
    consumer is Some;

class JiraModuleList extends CoreModuleList {
  Iterable<Page> get repoPages => filtered(JiraModuleTypes.repoPage);
  @Ensure(_oauthConsumerMandatoryForJira,
      description: 'oauth consumer is mandatory for jira')
  Option<OAuthConsumer> get oauthConsumer => super.oauthConsumer;

  JiraModuleList(
      {Iterable<Page> generalPages: const [],
      Iterable<Page> adminPages: const [],
      Iterable<WebItem> webItems: const [],
      Iterable<WebPanel> webPanels: const [],
      Page configurePage,
      Iterable<Webhook> webhooks: const [],
      OAuthConsumer oauthConsumer,
      Iterable<Page> repoPages})
      : super.fromModules(concat([
          CoreModuleList.createCoreModules(generalPages, adminPages, webItems,
              webPanels, configurePage, webhooks, oauthConsumer),
          BaseModuleList.mapModules(repoPages, JiraModuleTypes.repoPage)
        ]));

  JiraModuleList.fromModules(Iterable<ClassifiedModule> modules)
      : super.fromModules(modules);

  JiraModuleList.fromCore(
      {CoreModuleList coreModules, Iterable<Page> repoPages})
      : super.fromModules(concat([
          coreModules.modules,
          BaseModuleList.mapModules(repoPages, JiraModuleTypes.repoPage)
        ]));

  factory JiraModuleList.fromJson(Map json) {
    final p = parseJson(json);
    return new JiraModuleList(
        generalPages: p.list('generalPage', (v) => new Page.fromJson(v)),
        adminPages: p.list('adminPage', (v) => new Page.fromJson(v)),
        repoPages: p.list('repoPage', (v) => new Page.fromJson(v)),
        webItems: p.list('webItem', (v) => new WebItem.fromJson(v)),
        webPanels: p.list('webPanels', (v) => new WebPanel.fromJson(v)),
        webhooks: p.list('webhook', (v) => new Webhook.fromJson(v)),
        configurePage: p.single('configurePage', (v) => new Page.fromJson(v)));
  }

  Map toJson() => (buildJson
        ..addAll(super.toJson())
        ..add('repoPage', repoPages))
      .json;
}
