library ac_dart.descriptor_model.jira.webhoosk;

import 'package:atlassian_connect_models/src/descriptor/descriptor_model.dart';
import 'package:stuff/stuff.dart';

import '../descriptor_model.dart';

class JiraWebhookIdentifier extends WebhookEventIdentifier {
  JiraWebhookIdentifier._(String key) : super(key);

  JiraWebhookIdentifier._std(String group, [String name])
      : this._(name != null ? '${group}_$name' : group);

  JiraWebhookIdentifier._created(String group) : this._std(group, 'created');
  JiraWebhookIdentifier._updated(String group) : this._std(group, 'updated');
  JiraWebhookIdentifier._deleted(String group) : this._std(group, 'deleted');
}

final _JiraEvents jiraWebhooks = new _JiraEvents._();

const String _scopePrefix = 'org.jira.';

class _JiraEvents {
  final _IssueEvents issue = new _IssueEvents._();

  _JiraEvents._();
}

class _IssueEvents {
  static const String issue = 'jira:issue';
  final JiraWebhookIdentifier created =
      new JiraWebhookIdentifier._created(issue);
  final JiraWebhookIdentifier updated =
      new JiraWebhookIdentifier._updated(issue);
  final JiraWebhookIdentifier deleted =
      new JiraWebhookIdentifier._deleted(issue);

  _IssueEvents._();
}

abstract class JiraWebHookEvent implements Jsonable {
  final JiraUser user;
  final JiraIssue issue;

  factory JiraWebHookEvent.fromJson(Map json) = _JiraWebHookEvent.fromJson;
}

class _JiraWebHookEvent extends DynamicJson implements JiraWebHookEvent {
  JiraUser get user => parseSingle('user', (v) => new _JiraUser.fromJson(v));

  JiraIssue get issue =>
      parseSingle('issue', (v) => new _JiraIssue.fromJson(v));

  _JiraWebHookEvent.fromJson(Map<String, Object> json) : super.fromJson(json);
}

abstract class JiraUser implements Jsonable {
  final String emailAddress, name;
  final String webhookEvent;

  JiraUser(this.emailAddress, this.name,
      this.webhookEvent); // TODO: could be a JiraWebhookIdentifier
}

class _JiraUser extends DynamicJson implements JiraUser {
  _JiraUser.fromJson(Map<String, Object> json) : super.fromJson(json);
}

abstract class JiraIssue implements Jsonable {
  final String id, key, title;
  final JiraIssueFields fields;

  JiraIssue(this.id, this.key, this.title, this.fields);
}

class _JiraIssue extends DynamicJson implements JiraIssue {
  _JiraIssue.fromJson(Map<String, Object> json) : super.fromJson(json);

  JiraIssueFields get fields =>
      parseSingle('fields', (v) => new _JiraIssueFields.fromJson(v));
}

abstract class JiraIssueFields implements Jsonable {
  final JiraUser creator;
  final JiraUser reporter;
  final DateTime created;

  JiraIssueFields(this.creator, this.reporter, this.created);
}

class _JiraIssueFields extends DynamicJson implements JiraIssueFields {
  _JiraIssueFields.fromJson(Map<String, Object> json) : super.fromJson(json);

  JiraUser get creator =>
      parseSingle('creator', (v) => new _JiraUser.fromJson(v));

  JiraUser get reporter =>
      parseSingle('reporter', (v) => new _JiraUser.fromJson(v));

  DateTime get created => parseSingle('created', (v) => DateTime.parse(v));
}
