library ac_dart.host_model;

import 'package:constrain/constrain.dart';
import 'package:option/option.dart';
import 'package:path/path.dart' as path;
import 'package:stuff/stuff.dart';

class AddonHost {
  @NotNull()
  final String key;

  @NotNull()
  final Uri baseUrl;

  @NotNull()
  final Option<String> addonSecret;

  AddonHost(this.key, this.baseUrl, [String addonSecret])
      : this.addonSecret = new Option(addonSecret);
}

enum ProductType { jira, confluence, bitbucket, hipchat }

ProductType productTypeFromLiteral(String literal) => ProductType.values
    .firstWhere((type) => literalForProductType(type) == literal);

String literalForProductType(ProductType type) =>
    type.toString().replaceFirst('ProductType.', '');

class AcHostDetails implements Jsonable {
  static const String _consumerInfoPath =
      "/plugins/servlet/oauth/consumer-info";

  @NotNull()
  final String key;

  @NotNull()
  final ProductType productType;

  String get productTypeLiteral => literalForProductType(productType);

  @deprecated
  final String publicKey;

  @NotNull()
  final Uri baseUrl;

  Uri get consumerInfoUrl =>
      Uri.parse(path.join(baseUrl.origin, _consumerInfoPath));

  AcHostDetails(this.key, this.productType, this.publicKey, this.baseUrl);

  AcHostDetails.build(
      {this.key, this.productType, this.publicKey, this.baseUrl});

  String toString() => 'AcHostDetails[$key, $productType, $baseUrl]';

  AcHostDetails.fromJson(Map<String, Object> jsonMap)
      : this(jsonMap['key'], productTypeFromLiteral(jsonMap['productType']),
            jsonMap['publicKey'], Uri.parse(jsonMap['baseUrl']));

  Map toJson() => (buildJson
        ..add('key', key)
        ..add('productType', productType, (v) => literalForProductType(v))
        ..add('baseUrl', baseUrl)
        ..add('publicKey', publicKey))
      .json;
}

class AcHostRegistration implements Jsonable {
  @NotNull()
  final int id;

  @NotNull()
  final AcHostDetails hostDetails;

  @NotNull()
  final String sharedSecret;

  ClientCredentials get clientId =>
      new ClientCredentials(hostDetails.key, sharedSecret);

  AcHostRegistration(this.id, this.hostDetails, this.sharedSecret);

  AcHostRegistration.build({this.id, this.hostDetails, this.sharedSecret});

  AcHostRegistration.fromJson(Map<String, Object> jsonMap)
      : this(0, new AcHostDetails.fromJson(jsonMap['hostDetails']),
            jsonMap['sharedSecret']);

  Map toJson() => (buildJson
        ..add('id', id)
        ..add('hostDetails', hostDetails)
        ..add('sharedSecret', sharedSecret))
      .json;

  String toString() => 'AcHostRegistration[$id, $hostDetails]';
}

/// Represents the client application's credentials.
class ClientCredentials {
  /// The identifier used to identify this application to the server.
  final String identifier;

  /// The client secret used to identify this application to the server.
  final String secret;

  ClientCredentials(this.identifier, this.secret) {
    if (identifier == null) {
      throw new ArgumentError('Argument identifier may not be null.');
    }
  }
}
